﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Web;
using System.Web.Routing;
using Nop.Core;
using Nop.Core.Domain.Configuration;
using Nop.Core.Domain.Directory;
using Nop.Core.Domain.Orders;
using Nop.Core.Domain.Payments;
using Nop.Core.Plugins;
using Nop.Plugin.Payments.OmniKassa.Controllers;
using Nop.Plugin.Payments.OmniKassa.Domain;
using Nop.Services.Configuration;
using Nop.Services.Directory;
using Nop.Services.Localization;
using Nop.Services.Orders;
using Nop.Services.Payments;
using Nop.Web.Framework;
using System.Linq;


namespace Nop.Plugin.Payments.OmniKassa
{
    /// <summary>
    /// OmniKassa payment processor
    /// </summary>
    public class OmniKassaPaymentProcessor : BasePlugin, IPaymentMethod
    {
        #region Fields

        private readonly OmniKassaPaymentSettings _omniKassaPaymentSettings;
        private readonly ISettingService _settingService;
        private readonly IOrderService _orderService;
        private readonly IWebHelper _webHelper;
        private readonly IOrderTotalCalculationService _orderTotalCalculationService;
        private readonly ICurrencyService _currencyService;
        private readonly CurrencySettings _currencySettings;
        private readonly IWorkContext _workContext;
        private readonly ILocalizationService _localizationService;
        private readonly HttpContextBase _httpContext;
        private readonly IStoreContext _storeContext;

        private readonly List<KeyValuePair<string, string>> _supportedCurrencies = new List<KeyValuePair<string, string>>();    

        #endregion

        #region Ctor

        public OmniKassaPaymentProcessor(OmniKassaPaymentSettings omniKassaPaymentSettings,
            ISettingService settingService,
            IOrderService orderService,
            IWebHelper webHelper,
            IOrderTotalCalculationService orderTotalCalculationService,
            ICurrencyService currencyService,
            CurrencySettings currencySettings,
            ILocalizationService localizationService,
            IWorkContext workcontext,
            HttpContextBase httpContext,
            IStoreContext storeContext)
        {
            _omniKassaPaymentSettings = omniKassaPaymentSettings;
            _settingService = settingService;
            _orderService = orderService;
            _webHelper = webHelper;
            _orderTotalCalculationService = orderTotalCalculationService;
            _currencyService = currencyService;
            _currencySettings = currencySettings;
            _workContext = workcontext;
            _localizationService = localizationService;
            _httpContext = httpContext;
            _storeContext = storeContext;

            InitSupportedCurrencies();
        }

        #endregion

        #region Utilities

        private void InitSupportedCurrencies()
        {
            _supportedCurrencies.Add(new KeyValuePair<string, string>("EUR", "978"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("USD", "840"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("CHF", "756"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("GBP", "826"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("CAD", "124"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("JPY", "392"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("AUD", "036"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("NOK", "578"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("SEK", "752"));
            _supportedCurrencies.Add(new KeyValuePair<string, string>("DKK", "208"));
        }

        //OmniKassa supported currencies to numeric code (ISO-4217)
        private string GetCurrencyNumericCode()
        {
            Currency primaryStoreCurrency = _currencyService.GetCurrencyById(_currencySettings.PrimaryStoreCurrencyId);

            //Get the numeric currency code
            string supportedCurrency =
                _supportedCurrencies.FirstOrDefault(x => x.Key == primaryStoreCurrency.CurrencyCode.ToUpper()).Value;

            if (String.IsNullOrEmpty(supportedCurrency))
            {
                throw new NopException(_localizationService.GetResource("Plugins.Payments.OmniKassa.CurrencyNotSupported"));
            }

            return supportedCurrency;
        }

        //OmniKassa supports the ISO-4217 numeric codification, but only for some currencies
        //Look in the method GetCurrencyNumericCode() for the supported currencies.        
        //The Omnikassa amount is like this: 106,55 => 10655
        // All the supported OmniKassa currencies supports 2 decimal digits after the comma,
        // The YEN does NOT supports decimal digits (following the ISO-4217 standard) 
        private static string ToOmniKassaAmount(decimal amount, string currencyNumericCode)
        {
            string result;

            if (currencyNumericCode.ToUpper() == "YEN")
            {
                result = decimal.Truncate(amount).ToString(CultureInfo.InvariantCulture);
            }
            else
            {
                result = decimal.Truncate(amount * 100m).ToString(CultureInfo.InvariantCulture);
            }

            return result;
        }

        //OmniKassa supports only two languages (English or Dutch), if we detect Dutch we use it
        //otherwise we use english
        private string GetOmniKassaLanguage()
        {
            string result;

            if (_workContext.WorkingLanguage.LanguageCulture.StartsWith("nl-", StringComparison.InvariantCultureIgnoreCase))
            {
                result = "nl";
            }
            else
            {
                result = "en";
            }

            return result;
        }

        private string BuildPaymentMeanBrandList()
        {
            string value = String.Empty;

            // If the payment method is a user choice, we use it
            if (_omniKassaPaymentSettings.IsPaymentMethodUserChoice)
            {

                value = (string) _httpContext.Session["OmniKassa.SelectedPaymentMethod"];

                if (!string.IsNullOrEmpty(value))
                    return value;
            }

            List<string> paymentlist =
                _omniKassaPaymentSettings.OmniKassaServicesList.XmlDeserializeFromString<List<OmniKassaService>>()
                    .Where(o => o.Active)
                    .Select(x => x.Name)
                    .ToList();
             //   _omniKassaPaymentSettings.PaymentMeanBrandList.Split(new[] { "," }, StringSplitOptions.RemoveEmptyEntries).ToList();
            
            Currency primaryStoreCurrency = _currencyService.GetCurrencyById(_currencySettings.PrimaryStoreCurrencyId);

            Helpers.RemovePaymentsBrandIfNotEur(paymentlist, primaryStoreCurrency.CurrencyCode);

            // The following can only occours if the OmniKassa plugin is configured to use ONLY IDEAL and/or MINITIX and
            // the primary store currency is not EUR
            if (paymentlist.Count < 1)
            {
                throw new NopException(_localizationService.GetResource("Plugins.Payments.OmniKassa.PaymentBrandNotPresent"));
            }

            value = string.Join(",", paymentlist);

            return value;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Process a payment
        /// </summary>
        /// <param name="processPaymentRequest">Payment info required for an order processing</param>
        /// <returns>Process payment result</returns>
        public ProcessPaymentResult ProcessPayment(ProcessPaymentRequest processPaymentRequest)
        {
            return new ProcessPaymentResult { NewPaymentStatus = PaymentStatus.Pending };
        }

        /// <summary>
        /// Post process payment (used by payment gateways that require redirecting to a third-party URL)
        /// </summary>
        /// <param name="postProcessPaymentRequest">Payment info required for an order processing</param>
        public void PostProcessPayment(PostProcessPaymentRequest postProcessPaymentRequest)
        {
            string secretKey = _omniKassaPaymentSettings.SecretKey;
            string secretKeyVersion = _omniKassaPaymentSettings.SecretKeyVersion;
            string merchantId =_omniKassaPaymentSettings.MerchantId;
            string omniKassaUrl = _omniKassaPaymentSettings.OmniKassaUrl;
            string transactionRef = Guid.NewGuid().ToString().Replace("-", "");

            if (_omniKassaPaymentSettings.TestMode)
            {                
                merchantId = "002020000000001";                
                secretKey = "002020000000001_KEY1";
                secretKeyVersion = "1";

                omniKassaUrl = _omniKassaPaymentSettings.TestOmniKassaUrl;
            }

            string currencyNumericCode = GetCurrencyNumericCode();
            string amount = ToOmniKassaAmount(postProcessPaymentRequest.Order.OrderTotal, currencyNumericCode);
            string language = GetOmniKassaLanguage();
            string paymentMeanBrand = BuildPaymentMeanBrandList();

           //POST-velden
            string data = String.Format("amount={0}|currencyCode={1}|merchantId={2}|normalReturnUrl={3}|" +
                                        "transactionReference={4}|keyVersion={5}|customerLanguage={6}|automaticResponseUrl={7}|orderId={8}|paymentMeanBrandList={9}",
                           amount,
                           currencyNumericCode,
                           merchantId,
                           _omniKassaPaymentSettings.NormalReturnUrl,
                           transactionRef,
                           secretKeyVersion,                           
                           language,
                           _omniKassaPaymentSettings.AutomaticResponseUrl,
                           postProcessPaymentRequest.Order.Id,
                           paymentMeanBrand);
            
            // Calculate Seal-field 
            string seal = Helpers.EncodeMessage(data, secretKey);

            var remotePostHelper = new RemotePost {FormName = "OmniKassaForm", Url = omniKassaUrl};

            remotePostHelper.Add("Data", data);
            remotePostHelper.Add("InterfaceVersion", _omniKassaPaymentSettings.InterfaceVersion);
            remotePostHelper.Add("Seal", seal);

            //Save the order transactionID so we can get the order back later in the OmniKassaReturn method
            postProcessPaymentRequest.Order.AuthorizationTransactionId = transactionRef;  // used to store the transactionID.
            _orderService.UpdateOrder(postProcessPaymentRequest.Order);

            //Do the call
            remotePostHelper.Post();
        }

        /// <summary>
        /// Gets additional handling fee
        /// </summary>
        /// <returns>Additional handling fee</returns>
        public decimal GetAdditionalHandlingFee(IList<ShoppingCartItem> cart)
        {
            decimal additionalFee;
            bool additionalFeePercentage;
            
            if (_omniKassaPaymentSettings.IsPaymentMethodUserChoice)
            {
                var storeScope = _storeContext.CurrentStore.Id;

                string paymentMethod = (string) _httpContext.Session["OmniKassa.SelectedPaymentMethod"];

                additionalFee =
                    _settingService.GetSettingByKey<decimal>(String.Format("omnikassapaymentsettings.{0}additionalfee",
                        paymentMethod), storeId: storeScope);

                additionalFeePercentage =
                    _settingService.GetSettingByKey<bool>(String.Format("omnikassapaymentsettings.{0}additionalfeepercentage",
                        paymentMethod), storeId: storeScope);
            }
            else
            {
                additionalFee = _omniKassaPaymentSettings.AdditionalFee;
                additionalFeePercentage = _omniKassaPaymentSettings.AdditionalFeePercentage;
            }

            var result = this.CalculateAdditionalFee(_orderTotalCalculationService, cart, additionalFee, additionalFeePercentage);
            return result;
        }

        /// <summary>
        /// Captures payment
        /// </summary>
        /// <param name="capturePaymentRequest">Capture payment request</param>
        /// <returns>Capture payment result</returns>
        public CapturePaymentResult Capture(CapturePaymentRequest capturePaymentRequest)
        {
            var result = new CapturePaymentResult();
            result.AddError("Capture method not supported");
            return result;
        }

        /// <summary>
        /// Refunds a payment
        /// </summary>
        /// <param name="refundPaymentRequest">Request</param>
        /// <returns>Result</returns>
        public RefundPaymentResult Refund(RefundPaymentRequest refundPaymentRequest)
        {
            var result = new RefundPaymentResult();
            result.AddError("Refund method not supported");
            return result;
        }

        /// <summary>
        /// Voids a payment
        /// </summary>
        /// <param name="voidPaymentRequest">Request</param>
        /// <returns>Result</returns>
        public VoidPaymentResult Void(VoidPaymentRequest voidPaymentRequest)
        {
            var result = new VoidPaymentResult();
            result.AddError("Void method not supported");
            return result;
        }

        /// <summary>
        /// Process recurring payment
        /// </summary>
        /// <param name="processPaymentRequest">Payment info required for an order processing</param>
        /// <returns>Process payment result</returns>
        public ProcessPaymentResult ProcessRecurringPayment(ProcessPaymentRequest processPaymentRequest)
        {
            var result = new ProcessPaymentResult();
            result.AddError("Recurring payment not supported");
            return result;
        }

        /// <summary>
        /// Cancels a recurring payment
        /// </summary>
        /// <param name="cancelPaymentRequest">Request</param>
        /// <returns>Result</returns>
        public CancelRecurringPaymentResult CancelRecurringPayment(CancelRecurringPaymentRequest cancelPaymentRequest)
        {
            var result = new CancelRecurringPaymentResult();
            result.AddError("Recurring payment not supported");
            return result;
        }

        /// <summary>
        /// Gets a value indicating whether customers can complete a payment after order is placed but not completed (for redirection payment methods)
        /// </summary>
        /// <param name="order">Order</param>
        /// <returns>Result</returns>
        public bool CanRePostProcessPayment(Order order)
        {
            if (order == null)
                throw new ArgumentNullException("order");

            //payment status should be Pending
            if (order.PaymentStatus != PaymentStatus.Pending)
                return false;

            //let's ensure that at least 1 minute passed after order is placed
            if ((DateTime.UtcNow - order.CreatedOnUtc).TotalMinutes < 1)
                return false;

            return true;
        }
        
        /// <summary>
        /// Gets a route for provider configuration
        /// </summary>
        /// <param name="actionName">Action name</param>
        /// <param name="controllerName">Controller name</param>
        /// <param name="routeValues">Route values</param>
        public void GetConfigurationRoute(out string actionName, out string controllerName, out RouteValueDictionary routeValues)
        {
            actionName = "Configure";
            controllerName = "PaymentOmniKassa";
            routeValues = new RouteValueDictionary { { "Namespaces", "Nop.Plugin.Payments.OmniKassa.Controllers" }, { "area", null } };
        }

        /// <summary>
        /// Gets a route for payment info
        /// </summary>
        /// <param name="actionName">Action name</param>
        /// <param name="controllerName">Controller name</param>
        /// <param name="routeValues">Route values</param>
        public void GetPaymentInfoRoute(out string actionName, out string controllerName, out RouteValueDictionary routeValues)
        {
            actionName = "PaymentInfo";
            controllerName = "PaymentOmniKassa";
            routeValues = new RouteValueDictionary { { "Namespaces", "Nop.Plugin.Payments.OmniKassa.Controllers" }, { "area", null } };
        }

        public Type GetControllerType()
        {
            return typeof(PaymentOmniKassaController);
        }

        public override void Install()
        {
            Currency primaryStoreCurrency = _currencyService.GetCurrencyById(_currencySettings.PrimaryStoreCurrencyId);

            // Firtst of all we check if the primary store currency is supported
            if (!_supportedCurrencies.Exists(x=> x.Key == primaryStoreCurrency.CurrencyCode.ToUpper()))
            {
                throw new NopException(
                    "Could not install the OmniKassa plugin, because the primary store currency is not supported");
            }

            string storeUrl = _webHelper.GetStoreLocation(false);

            List<OmniKassaService> oks = new List<OmniKassaService>();
            // If the primary store currency is EUR we set IDEAL as default
            // otherwise we set VISA and MASTERCARD
            oks.Add(new OmniKassaService(Constants.Ideal) { Active = primaryStoreCurrency.CurrencyCode == "EUR", AdditionalFee = 0, AdditionalFeePercentage = false });
            oks.Add(new OmniKassaService(Constants.Visa) { Active = primaryStoreCurrency.CurrencyCode != "EUR", AdditionalFee = 0, AdditionalFeePercentage = false });
            oks.Add(new OmniKassaService(Constants.MasterCard) { Active = primaryStoreCurrency.CurrencyCode != "EUR", AdditionalFee = 0, AdditionalFeePercentage = false });
            oks.Add(new OmniKassaService(Constants.Maestro) { Active = primaryStoreCurrency.CurrencyCode != "EUR", AdditionalFee = 0, AdditionalFeePercentage = false });
            oks.Add(new OmniKassaService(Constants.MiniTix) { Active = false, AdditionalFee = 0, AdditionalFeePercentage = false });
            oks.Add(new OmniKassaService(Constants.Acceptgiro) { Active = false, AdditionalFee = 0, AdditionalFeePercentage = false });
            oks.Add(new OmniKassaService(Constants.Incasso) { Active = false, AdditionalFee = 0, AdditionalFeePercentage = false });
            oks.Add(new OmniKassaService(Constants.Rembours) { Active = false, AdditionalFee = 0, AdditionalFeePercentage = false });
            
            var settings = new OmniKassaPaymentSettings
            {
                TestMode = true,
                NormalReturnUrl = storeUrl + "Plugins/PaymentOmniKassa/OmniKassaReturn",
                AutomaticResponseUrl = storeUrl + "Plugins/PaymentOmniKassa/OmniKassaReport",
                OmniKassaUrl = "https://payment-webinit.omnikassa.rabobank.nl/paymentServlet",
                TestOmniKassaUrl = "https://payment-webinit.simu.omnikassa.rabobank.nl/paymentServlet",                          
                MinimumAmount = 0.01m,                                                                     
                InterfaceVersion = "HP_1.0",
                AdditionalFee = 0,
                AdditionalFeePercentage = false,
                OmniKassaServicesList = oks.XmlSerializeToString(),
                RedirectionTip = "You will be redirected to the OmniKassa site to complete the order payment.",
                AcceptgiroDefaultDisplayName = "Acceptgiro",
                IdealDefaultDisplayName = "iDeal",
                IncassoDefaultDisplayName = "Incasso",
                MaestroDefaultDisplayName = "Maestro",
                MasterCardDefaultDisplayName = "MasterCard",
                MiniTixDefaultDisplayName = "MiniTix",
                RemboursDefaultDisplayName = "Rembours",
                VisaDefaultDisplayName = "Visa"
              };

            _settingService.SaveSetting(settings);

            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.RedirectionTipLabel", "RedirectionTip");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.RedirectionTipLabel.Hint", "Text displayed to the user during checkout process");         
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.URL", "OmniKassa URL");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.URL.Hint", "The OmniKassa live URL");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestURL", "OmniKassa TEST URL");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestURL.Hint", "OmniKassa TEST URL");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MerchantID", "Merchant ID");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MerchantID.Hint", "Your Merchant ID");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.ReturnURL", "Normal Return Url");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.ReturnURL.Hint", "The url where the user is redirected after the payment at OmniKassa, if he/she clicks continue");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AutomaticResponseURL", "Automatische Response Url");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AutomaticResponseURL.Hint", "The url that OmniKassa will always call after a payment");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.InterfaceVersion", "Interface Version");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.InterfaceVersion.Hint", "Interface Version");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AdditionalFee", "Additional Fee");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AdditionalFee.Hint", "The additional fee using this payment method");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AdditionalFeePercentage", "Additional Fee. Use percentage");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AdditionalFeePercentage.Hint", "Determines whether to apply a percentage additional fee to the order total. If not enabled, a fixed value is used.");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MinimumAmount", "Minimum Order Amount");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MinimumAmount.Hint", "Minimum Order Amount");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKey", "Secret Key");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKey.Hint", "The Secret Key you got from OmniKassa");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKeyVersion", "Secret Key Version");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKeyVersion.Hint", "The Secret Key Version number you got from OmniKassa");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.PaymentMeanBrandList", "Payment Mean Brand List");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.PaymentMeanBrandList.Hint", "Choose the available Payments Methods for your customers");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestMode", "Test Mode");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestMode.Hint", "Check to enable test mode");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.URL.Required", "OmniKassa URL is required when not in test mode");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKey.Required", "The secret key is required when not in test mode");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKeyVersion.Required", "The secret key version is required when not in test mode");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MerchantID.Required", "Merchant ID is required when not in test mode");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.ReturnURL.Required", "Return URL is required when not in test mode");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AutomaticResponseURL.Required", "Automatic Response URL is required when not in test mode");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.InterfaceVersion.Required", "The interface version is required");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestURL.Required", "In Test mode is the Test URL  required");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.EditRedirectionTip", "Edit RedirectionTip");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AvailablePaymentMeanBrandList", "At least one Payment Mean Brand MUST be active");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.PaymentMethodUserChoice", "Payment Method User Choice");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.PaymentMethodUserChoice.Hint", "Check this if you want that users have to choose the Omnikassa payment method during checkout, at the payment info screen (step 5)" );
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.IdealLabel", "IDEAL name");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.IdealLabel.Hint", "IDEAL name displayed to the user if Payment Method User Choice is enabled");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.VisaLabel", "VISA name");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.VisaLabel.Hint", "VISA name displayed to the user if Payment Method User Choice is enabled");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MasterCardLabel", "MASTERCARD name");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MasterCardLabel.Hint", "MASTERCARD name displayed to the user if Payment Method User Choice is enabled");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MaestroLabel", "MAESTRO name");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MaestroLabel.Hint", "MAESTRO name displayed to the user if Payment Method User Choice is enabled");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MiniTixLabel", "MINITIX name");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MiniTixLabel.Hint", "MINITIX name displayed to the user if Payment Method User Choice is enabled");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.IncassoLabel", "INCASSO name");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.IncassoLabel.Hint", "INCASSO name displayed to the user if Payment Method User Choice is enabled");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.RemboursLabel", "REMBOURS name");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.RemboursLabel.Hint", "REMBOURS name displayed to the user if Payment Method User Choice is enabled");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AcceptgiroLabel", "ACCPETGIRO name");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AcceptgiroLabel.Hint", "ACCEPTGIRO name displayed to the user if Payment Method User Choice is enabled");

            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentBrandNotPresent", "There is any PaymentBrand available");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentFailedTitle", "Payment Failed");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentFailedMessage", "The payment for the following order has failed");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.OrderNumber", "Order Number");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.CurrencyNotSupported", "The primary store currency is not supported");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentChoice.Required", "Please choose a payment method");
            this.AddOrUpdatePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentUserChoiceTitle", "Choose how do you want to pay");
         
            base.Install();
        }

        public override void Uninstall()
        {
            _settingService.DeleteSetting<OmniKassaPaymentSettings>();

            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.RedirectionTipLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.RedirectionTipLabel.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.URL");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.URL.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestURL");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestURL.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MerchantID");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MerchantID.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.ReturnURL");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.ReturnURL.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AutomaticResponseURL");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AutomaticResponseURL.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.InterfaceVersion");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.InterfaceVersion.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AdditionalFee");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AdditionalFee.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AdditionalFeePercentage");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AdditionalFeePercentage.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MinimumAmount");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MinimumAmount.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKey");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKey.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKeyVersion");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKeyVersion.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.PaymentMeanBrandList");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.PaymentMeanBrandList.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestMode");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestMode.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.URL.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKey.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.SecretKeyVersion.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MerchantID.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.ReturnURL.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AutomaticResponseURL.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.InterfaceVersion.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.TestURL.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.EditRedirectionTip");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AvailablePaymentMeanBrandList");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.PaymentMethodUserChoice");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.PaymentMethodUserChoice.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.IdealLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.IdealLabel.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.VisaLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.VisaLabel.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MasterCardLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MasterCardLabel.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MaestroLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MaestroLabel.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MiniTixLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.MiniTixLabel.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.IncassoLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.IncassoLabel.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.RemboursLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.RemboursLabel.Hint");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AcceptgiroLabel");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.Admin.AcceptgiroLabel.Hint");

            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentChoice.Required");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentUserChoiceTitle");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentBrandNotPresent");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentFailedTitle");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.PaymentFailedMessage");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.OrderNumber");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.CurrencyNotSupported");

            //Clean also all the localized resources 
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.RedirectionTip.Localized");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.IdealDisplayName.Localized");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.MasterCardDisplayName.Localized");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.VisaDisplayName.Localized");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.MaestroDisplayName.Localized");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.MiniTixDisplayName.Localized");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.IncassoDisplayName.Localized");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.RemboursDisplayName.Localized");
            this.DeletePluginLocaleResource("Plugins.Payments.OmniKassa.AcceptgiroDisplayName.Localized");

            base.Uninstall();
        }

        #endregion

        #region Properies

        /// <summary>
        /// Gets a value indicating whether capture is supported
        /// </summary>
        public bool SupportCapture
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets a value indicating whether partial refund is supported
        /// </summary>
        public bool SupportPartiallyRefund
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets a value indicating whether refund is supported
        /// </summary>
        public bool SupportRefund
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets a value indicating whether void is supported
        /// </summary>
        public bool SupportVoid
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets a recurring payment type of payment method
        /// </summary>
        public RecurringPaymentType RecurringPaymentType
        {
            get
            {
                return RecurringPaymentType.NotSupported;
            }
        }

        /// <summary>
        /// Gets a payment method type
        /// </summary>
        public PaymentMethodType PaymentMethodType
        {
            get
            {
                return PaymentMethodType.Redirection;
            }
        }

        #endregion
    }
}